
# q-expansion of newform 4550.2.a.m, downloaded from the LMFDB on 08 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4550
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [2], [0], [1], [2], [1], [-2], [0], [8], [-2], [4], [-2], [0], [8], [4], [-4], [4], [2], [12], [-2], [-10], [0], [4], [4], [14], [-6], [10], [0], [-6], [4], [0], [12], [6], [4], [-18], [-2], [14], [20], [0], [-6], [-24], [-18], [20], [2], [22], [16], [0], [8], [28], [20], [-24], [-6], [4], [-20], [-10], [12], [-6], [-24], [-16], [-6], [-14], [-2], [32], [-24], [2], [-22], [10], [28], [36], [32], [-14], [-30], [-10], [-32], [30], [12], [10], [-30], [-10], [-20], [-36], [34], [-14], [-14], [32], [-24], [-38], [-22], [0], [32], [-18], [24], [0], [20], [-10], [-2], [44], [6], [22], [-10], [-8], [-10], [-42], [-10], [-32], [-14], [24], [18], [36], [-22], [30], [34], [-6], [8], [46], [2], [40], [10], [-36], [0], [16], [4], [6], [-52], [-36], [30], [30], [-24], [14], [-46], [-34], [16], [4], [-16], [48], [-48], [-26], [32], [-30], [6], [4], [-22], [-8], [-36], [-22], [-40], [-34], [-22], [12], [0], [22], [26], [20], [-30], [4], [0], [-16], [-8], [-50], [24], [-60], [-36], [-16], [28], [-18], [-16], [-28], [18], [22], [-30], [62], [4], [-12], [2], [32], [-50], [-38], [-6], [-42], [-30], [-24], [-12], [56], [-54], [40], [-6], [-20], [-24], [-42], [-44], [-2], [-12], [-48], [-36], [-32], [66], [58], [24], [-18], [26], [10], [16], [-6], [-38], [12], [-46], [56], [-12], [-66], [-52], [-40], [2], [64], [-36], [28], [-8], [-4], [-38], [32], [46], [-38], [-36], [-2], [32], [-50], [14], [6], [-4], [-62], [-22], [-8], [28], [-50], [54], [-2], [-24], [-28], [18], [-68], [24], [-36], [-36], [42], [-54], [30], [-16], [4], [36], [-58], [-28], [6], [-36], [-48], [-6], [6], [-30], [-2], [52], [62], [42], [0], [-64], [-24], [46], [-30], [60], [18], [-12], [-68], [-44], [58], [28], [26], [40], [62], [-4], [36], [10], [-10], [36], [14], [2], [26], [-16], [4], [-40], [64], [62], [64], [6], [-78], [-26], [-18], [0], [-8], [22], [16], [6], [4], [60], [78], [82], [-36], [-36], [8], [70], [-56], [66], [-18], [36], [14], [70], [58], [-52], [8], [-22], [8], [34], [62], [18], [52], [38], [-22], [-72], [-66], [6], [6], [28], [20], [-36], [10], [28], [16], [32], [-42], [60], [-22], [-14], [64], [-22], [-46], [-56], [16], [-52], [34], [12], [94], [-60], [-8], [-94], [-20], [-58], [18], [46], [-86], [6], [-12], [-72], [60], [72], [-62], [18], [24], [58], [66], [0], [18], [0], [-10], [84], [36], [48], [98], [-48], [-68], [60], [64], [-34], [10], [-38], [20], [-74], [-64], [22], [12], [12], [-46], [38], [-44], [78], [46], [-22], [-30], [22], [18], [4], [-76], [54], [44], [30], [76], [80], [-88], [86], [-60], [64], [-62], [42], [-32], [-48], [-2], [40], [24], [-64], [48], [22], [18], [62], [-74], [-34], [70], [54], [-8], [36], [86], [-42], [36], [-10], [88], [-88]]
